# key chars

COMMENT_MARK = "#"
SECTION_IN = "["
SECTION_OUT = "]"
PARSER_SECTION = "@"
BLANK = " "
EQ = "="
ARROW = ">"

DICT_ENTRY = "@dict"
LIST_ENTRY = "@list"
JSON_ENTRY = "@json"

# section types codes
SECTION_TYPES = dict(
    STANDARD=0,
    DIRTREE=1,
    SYMLINKS=2,
    JSON=3
)



import json

from big_config.path_list import PathsList
from constants import *


class ParserException(Exception):
    def __init__(self, message):
        self.message = "Parser exception." + message
        Exception.__init__(self, self.message)


class Parsers(object):
    def __init__(self, silent=False, warnings=None):
        self.silent = silent
        self.warnings = warnings if warnings else []

    @staticmethod
    def typed_value(entry):
        # is it an integer?
        try:
            return int(entry)
        except ValueError:
            pass
        # is it a float?
        try:
            return float(entry)
        except ValueError:
            pass
        # is it a boolean value ?
        if entry.lower() == "yes" or entry.lower() == "true":
            return True
        if entry.lower() == "no" or entry.lower() == "false":
            return False
        # if it's a json
        if entry[0] in ["{", "["] and entry[-1] in ["}", "]"]:
            try:
                entry = json.loads(entry)
            except (AttributeError, ValueError):
                # then it's definitely a string
                return entry
        # is a json parsed value
        return entry

    @staticmethod
    def get_indent(line):
        return len(line) - len(line.lstrip())

    def parse_line(self, root, line):
        bits = line.split(EQ)
        root[bits[0].strip()] = self.typed_value(bits[1].strip())

    def parse_dict_entry(self, root, raw):
        entry = raw[0].strip().split(BLANK)[1].strip()
        indent = self.get_indent(raw[0])
        del raw[0]
        root[entry] = dict()
        while len(raw) and self.get_indent(raw[0]) > indent:
            self.parse_line(root[entry], raw[0])
            del raw[0]

    def parse_json_entry(self, root, raw):
        entry = raw[0].strip().split(BLANK)[1].strip()
        indent = self.get_indent(raw[0])
        del raw[0]
        raw_json = ""
        while len(raw) and self.get_indent(raw[0]) > indent:
            raw_json += raw[0]
            del raw[0]
        try:
            root[entry] = json.loads(raw_json)
        except (AttributeError, ValueError):
            message = "Json entry parse error for entry: %s." % entry
            self.warnings.append("Parser error. " + message)
            if not self.silent:
                raise ParserException(message)
            root[entry] = {}

    def parse_json_section(self, raw, name):
        try:
            return json.loads("".join(raw))
        except (AttributeError, ValueError):
            message = "Json parse error in \"%s\" section." % name
            self.warnings.append("Parser error. " + message)
            if not self.silent:
                raise ParserException(message)
        return {}

    def parse_list_entry(self, root, raw):
        entry = raw[0].strip().split(BLANK)[1].strip()
        indent = self.get_indent(raw[0])
        del raw[0]
        root[entry] = []
        while len(raw) and self.get_indent(raw[0]) > indent:
            root[entry].append(self.typed_value(raw[0].strip()))
            del raw[0]

    @staticmethod
    def parse_symlink_list(raw):
        output = []
        while len(raw):
            while len(raw) and ARROW in raw[0]:
                entry = raw[0].split(ARROW)
                output.append({'source': entry[0].strip(), 'dest': entry[1].strip()})
                del raw[0]
        return output

    @staticmethod
    def parse_dir_list(raw):
        return PathsList(raw)

    def parse(self, raw):
        output = dict()
        while len(raw):
            while len(raw) and EQ in raw[0]:
                self.parse_line(output, raw[0])
                del raw[0]
            if len(raw) and DICT_ENTRY in raw[0]:
                self.parse_dict_entry(output, raw)
            if len(raw) and LIST_ENTRY in raw[0]:
                self.parse_list_entry(output, raw)
            if len(raw) and JSON_ENTRY in raw[0]:
                self.parse_json_entry(output, raw)
        return output



import unittest
from path_list import PathsList

class PathsListTest(unittest.TestCase):
    def test_one_step(self):
        test_arr = """
                global
                media
                    video
                        video-sources
                        picture-sources
                        vector-sources
                        audio-sources
                        font-sources
                        document-sources
                    audio
                        music&soundtrack
                        sfx
                        vo
            """.split("\n")
        paths_list = PathsList(test_arr)
        self.assertEqual(
            paths_list,
            [
                'global',
                'media',
                'media/video',
                'media/video/video-sources',
                'media/video/picture-sources',
                'media/video/vector-sources',
                'media/video/audio-sources',
                'media/video/font-sources',
                'media/video/document-sources',
                'media/audio',
                'media/audio/music&soundtrack',
                'media/audio/sfx',
                'media/audio/vo'
            ]
        )



import logging
import unittest

import os
from testfixtures import LogCapture

from config import Config, ConfigException


class ConfigTests(unittest.TestCase):
    def setUp(self):
        self.cwd = os.path.dirname(os.path.realpath(__file__))
        self.fail_exists_file = os.path.join(self.cwd, "fixtures", "test_exists_fail.conf")
        self.fail_read_file = os.path.join(self.cwd, "fixtures", "test_read_fail.conf")
        self.fail_content_file = os.path.join(self.cwd, "fixtures", "test_content_fail.conf")
        self.fail_json_section_file = os.path.join(self.cwd, "fixtures", "test_json_section_fail.conf")
        self.pass_schema_test = os.path.join(self.cwd, "fixtures", "test_schema_pass.conf")
        with open(self.fail_read_file, "w+") as fh:
            fh.write(self.fail_read_file)
        os.chmod(self.fail_read_file, 0o300)

    def tearDown(self):
        if os.path.isfile(self.fail_read_file):
            os.remove(self.fail_read_file)

    def test_pre_parse_pass(self):
        test_str = """
            # test config file
              # test comment strip function

            this  be left out

            [standard_section]

                # inner comment
                standard body line1
                standard body line2

                standard body line3

                [@DIRTREE test_paths]

                dirtree body line1

                # inner comment
                dirtree body line2
                dirtree body line3
        """
        cfg = Config()
        raw_data = cfg.pre_parse(test_str)
        self.assertEqual(raw_data[0]["name"], "standard_section")
        self.assertEqual(raw_data[0]["type"], 0)
        self.assertEqual(raw_data[0]["body"], [
            "                standard body line1",
            "                standard body line2",
            "                standard body line3"
        ])
        self.assertEqual(raw_data[1]["name"], "test_paths")
        self.assertEqual(raw_data[1]["type"], 1)
        self.assertEqual(raw_data[1]["body"], [
            "                dirtree body line1",
            "                dirtree body line2",
            "                dirtree body line3"
        ])

    def test_pre_parse_fail_section_silent(self):
        test_str = """
            [standard_section]
                standard body line1
                standard body line2
                standard body line3

            [@FAIL fail_section]
                fail body line1
                fail body line2
        """
        cfg = Config(silent=True)
        raw_data = cfg.pre_parse(test_str)
        self.assertEqual(raw_data[0]["name"], "standard_section")
        self.assertEqual(raw_data[0]["type"], 0)
        self.assertEqual(raw_data[0]["body"], [
            "                standard body line1",
            "                standard body line2",
            "                standard body line3"
        ])
        self.assertEqual(len(raw_data), 1)
        self.assertEqual(cfg.warnings()[0], "Config Exception. Key error raised in section type: FAIL")
        self.assertEqual(len(cfg.warnings()), 1)

    def test_pre_parse_fail_section_raise(self):
        test_str = """
            [standard_section]
                standard body line1
                standard body line2
                standard body line3

            [@FAIL fail_section]
                fail body line1
                fail body line2
        """
        with LogCapture() as l:
            logger = logging.getLogger()
            cfg = Config(logger=logger)
            self.assertRaises(ConfigException, cfg.pre_parse, test_str)
            self.assertTrue("Key error raised in section type: fail_section", l.records[0].msg)
            self.assertTrue(len(l.records) == 1)

    def test_read_config_fail_raise(self):
        with LogCapture() as l:
            logger = logging.getLogger()
            cfg = Config(logger=logger)
            self.assertRaises(
                ConfigException,
                cfg.read_config,
                os.path.join(self.cwd, "fixtures", self.fail_exists_file)
            )
            self.assertEqual(len(l.records), 1)
            self.assertTrue(self.fail_exists_file in l.records[0].msg)

    def test_read_config_file_fail_silent(self):
        cfg = Config(silent=True)
        self.assertEqual(cfg.read_config(self.fail_exists_file), "")
        self.assertEqual(
            cfg.warnings()[0],
            "Config error. " + self.fail_exists_file + " missing configuration file."
        )
        self.assertEqual(len(cfg.warnings()), 1)

    def test_read_config_file_fail_raise(self):
        with LogCapture() as l:
            logger = logging.getLogger()
            self.assertRaises(ConfigException, Config, os.path.join(self.cwd, "fixtures", self.fail_read_file), logger)
            self.assertEqual(len(l.records), 1)
            self.assertTrue(self.fail_read_file in l.records[0].msg)
            self.assertTrue("Permission denied" in l.records[0].msg)

    def test_read_config_content_fail_silent(self):
        cfg = Config(silent=True)
        self.assertEqual(cfg.read_config(self.fail_read_file), "")
        self.assertEqual(
            cfg.warnings()[0],
            "Configuration error for file: " + self.fail_read_file + " -  Permission denied."
        )
        self.assertEqual(len(cfg.warnings()), 1)

    def test_content_config_fail_raise(self):
        with LogCapture() as l:
            logger = logging.getLogger()
            self.assertRaises(ConfigException, Config, self.fail_content_file, logger)
            self.assertEqual(len(l.records), 1)
            self.assertTrue(self.fail_content_file + " is empty" in l.records[0].msg)

    def test_content_config_no_output_fail_raise(self):
        test_str = \
            """
                no section line1
                no section line2
            """
        with LogCapture() as l:
            logger = logging.getLogger()
            cfg = Config(logger=logger)
            self.assertRaises(ConfigException, cfg.pre_parse, test_str)
            self.assertTrue("Config error. Could not parse file content. Got empty dict", l.records[0].msg)
            self.assertTrue(len(l.records) == 1)

    def test_json_section_fail_raise(self):
        with LogCapture() as l:
            logger = logging.getLogger()
            self.assertRaises(ConfigException, Config, self.fail_json_section_file, logger)
            self.assertEqual(len(l.records), 1)
            self.assertTrue('Parser exception.Json parse error in "json_fail" section.' in l.records[0].msg)

    def test_json_section_fail_silent(self):
        cfg = Config(self.fail_json_section_file, silent=True)
        self.assertEqual(cfg, {
            'json_fail': {},
            'standard_section1': {
                'sec1_key1': 'sec1_value1',
                'sec1_key3': 'sec1_value3',
                'sec1_key2': 'sec1_value2'
            },
            'standard_section2': {
                'sec2_key1': 'sec2_value1',
                'sec2_key2': 'sec2_value2',
                'sec2_key3': 'sec2_value3'
            }
        })
        self.assertEqual(
            cfg.warnings()[0],
            "Parser error. Json parse error in \"json_fail\" section."
        )
        self.assertEqual(len(cfg.warnings()), 1)

    def test_config_data_pass(self):
        cfg = Config(os.path.join(self.cwd, "fixtures", "test1.conf"))
        self.assertEqual(cfg["standard_section"], dict(
            test_int=1234,
            test_float=12.34,
            test_bool=True,
            test_string="string",
            dict_entry=dict(
                key1="value1",
                key2="value2"
            ),
            test_key="test_value",
            list_entry=["v1", 1234, True],
            test_json_entry={u'key21': u'value21', u'ke1': {u'key12': u'value12', u'key11': u'value11'}},
            test_json_list=[u'item1', u'item2', {u'key1': u'value1'}],
            test_json_line={u'key21': u'value21', u'ke1': {u'key12': u'value12', u'key11': u'value11'}}
        ))
        self.assertEqual(cfg["test_json_section"],
                         {u'key21': u'value21', u'ke1': {u'key12': u'value12', u'key11': u'value11'}})
        self.assertEqual(cfg["test_paths"],
                         [
                             'dir1',
                             'dir1/dir11',
                             'dir1/dir12',
                             'dir1/dir12/dir121',
                             'dir2',
                             'dir2/dir21',
                             'dir2/dir22',
                             'dir2/dir23'
                         ])
        self.assertEqual(cfg["sym_list"],
                         [
                             {'dest': 'dest_root/dir1', 'source': 'source_root/dir1'},
                             {'dest': 'dest_root/dir2/dir21', 'source': 'source_root/dir2/dir21'},
                             {'dest': 'dest_root/dir3', 'source': 'source_root/dir3'}
                         ])

    def test_schema_pass(self):
        conf_schema = {
            "standard_section": {
                "type": "dict",
                "schema": {
                    "test_int": {
                        "type": "integer",
                        "required": True
                    },
                    "test_float": {
                        "type": "float",
                        "required": True
                    },
                    "test_bool": {
                        "type": "boolean",
                        "required": True
                    },
                    "test_string": {
                        "type": "string",
                        "required": True
                    },
                    "test_dict": {
                        "type": "dict",
                        "schema": {
                            "key1": {
                                "type": "string",
                                "required": True
                            },
                            "key2": {
                                "type": "string",
                                "required": True
                            }
                        }
                    },
                    "test_list": {
                        "type": "list",
                        "schema": {
                            "allow_unknown": True
                        }
                    }
                }
            }
        }
        cfg = Config(schema=conf_schema)
        self.assertEqual(
            cfg.initialize(self.pass_schema_test),
            {
                'standard_section': {
                    'test_dict': {
                        'key2': 'value2',
                        'key1': 'value1'
                    },
                    'test_bool': True,
                    'test_list': ['v1', 1234, True],
                    'test_float': 12.34,
                    'test_int': 1234,
                    'test_string': 'string'
                }
            }
        )
        self.assertEqual(len(cfg.warnings()), 0)

    def test_schema_fail_raise(self):
        conf_schema = {
            "standard_section": {
                "type": "dict",
                "schema": {
                    "test_int": {
                        "type": "string",  # this will fail
                        "required": True
                    },
                    "test_float": {
                        "type": "float",
                        "required": True
                    },
                    "test_bool": {
                        "type": "boolean",
                        "required": True
                    },
                    "test_string": {
                        "type": "string",
                        "required": True
                    },
                    "test_dict": {
                        "type": "dict",
                        "schema": {
                            "key1": {
                                "type": "string",
                                "required": True
                            },
                            "key2": {
                                "type": "string",
                                "required": True
                            }
                        }
                    },
                    "test_list": {  # this will fail too
                        "type": "integer",
                    }
                }
            }
        }
        with LogCapture() as l:
            logger = logging.getLogger()
            cfg = Config(schema=conf_schema, logger=logger)
            self.assertRaises(ConfigException, cfg.initialize, self.pass_schema_test)
            self.assertEqual(len(l.records), 1)
            self.assertEqual(
                'Configuration validation error(s): \n{\n   "standard_section": ' +
                '{\n      "test_int": "must be of string type", ' +
                '\n      "test_list": "must be of integer type"\n   }\n}',
                l.records[0].msg
            )

    def test_schema_fail_silent(self):
        conf_schema = {
            "standard_section": {
                "type": "dict",
                "schema": {
                    "test_int": {
                        "type": "string",  # this will fail
                        "required": True
                    },
                    "test_float": {
                        "type": "float",
                        "required": True
                    },
                    "test_bool": {
                        "type": "boolean",
                        "required": True
                    },
                    "test_string": {
                        "type": "string",
                        "required": True
                    },
                    "test_dict": {
                        "type": "dict",
                        "schema": {
                            "key1": {
                                "type": "string",
                                "required": True
                            },
                            "key2": {
                                "type": "string",
                                "required": True
                            }
                        }
                    },
                    "test_list": {  # this will fail too
                        "type": "integer",
                    }
                }
            }
        }
        cfg = Config(schema=conf_schema, silent=True)
        self.assertEqual(cfg.initialize(self.pass_schema_test), {})
        self.assertEqual(len(cfg.warnings()), 1)
        self.assertEqual(
            'Configuration validation error(s): \n{\n   "standard_section": ' +
            '{\n      "test_int": "must be of string type", ' +
            '\n      "test_list": "must be of integer type"\n   }\n}',
            cfg.warnings()[0]
        )



import unittest

import os

from parsers import Parsers, ParserException


class ParsersTest(unittest.TestCase):
    def setUp(self):
        self.cwd = os.path.dirname(os.path.realpath(__file__))

    def test_typed_value(self):
        self.assertEqual(Parsers.typed_value("1234"), 1234)
        self.assertEqual(Parsers.typed_value("12.34"), 12.34)
        self.assertEqual(Parsers.typed_value("True"), True)
        self.assertEqual(Parsers.typed_value("false"), False)
        self.assertEqual(Parsers.typed_value("yes"), True)
        self.assertEqual(Parsers.typed_value("yEs"), True)
        self.assertEqual(Parsers.typed_value("tRue"), True)
        self.assertEqual(Parsers.typed_value('{"key" : "value"}'), {"key": "value"})
        self.assertEqual(Parsers.typed_value('["value1", "value2"]'), ["value1", "value2"])

    def test_line_parser(self):
        pars = Parsers()
        test_root = dict()
        pars.parse_line(test_root, "test_int = 1234")
        self.assertEqual(test_root["test_int"], 1234)
        pars.parse_line(test_root, "test_float = 12.34")
        self.assertEqual(test_root["test_float"], 12.34)

    def test_dict_parser(self):
        pars = Parsers()
        test_root = dict()
        # stop if raw ends
        raw = [
            "@dict test_dict",
            "  t_int = 1234",
            "    t_float = 12.34",
            " t_yes = yEs",
            "     t_false = faLse",
            "  t_string = string"
        ]
        pars.parse_dict_entry(test_root, raw)
        self.assertEqual(test_root["test_dict"]["t_int"], 1234)
        self.assertEqual(test_root["test_dict"]["t_float"], 12.34)
        self.assertEqual(test_root["test_dict"]["t_yes"], True)
        self.assertEqual(test_root["test_dict"]["t_false"], False)
        self.assertEqual(test_root["test_dict"]["t_string"], "string")

        # stop on next upper key
        raw = [
            "@dict test_dict",
            "  t_int = 1234",
            "    t_float = 12.34",
            " t_yes = yEs",
            "     t_false = faLse",
            "  t_string = string",
            "key_error = raised"
        ]

        def get_error_key(dic, key):
            return dic[key]

        pars.parse_dict_entry(test_root, raw)
        self.assertEqual(test_root["test_dict"]["t_int"], 1234)
        self.assertEqual(test_root["test_dict"]["t_float"], 12.34)
        self.assertEqual(test_root["test_dict"]["t_yes"], True)
        self.assertEqual(test_root["test_dict"]["t_false"], False)
        self.assertEqual(test_root["test_dict"]["t_string"], "string")
        self.assertRaises(KeyError, get_error_key, test_root["test_dict"], "key_error")

    def test_list_parser(self):
        pars = Parsers()
        test_root = dict()
        # stop if raw ends
        raw = [
            "@dict test_dict",
            "  1234",
            "    12.34",
            " yEs",
            "     faLse",
            "  string"
        ]
        pars.parse_list_entry(test_root, raw)
        self.assertEqual(test_root["test_dict"][0], 1234)
        self.assertEqual(test_root["test_dict"][1], 12.34)
        self.assertEqual(test_root["test_dict"][2], True)
        self.assertEqual(test_root["test_dict"][3], False)
        self.assertEqual(test_root["test_dict"][4], "string")

        # stop on next upper key
        raw = [
            "@dict test_dict",
            "  1234",
            "    12.34",
            " yEs",
            "     faLse",
            "  string",
            "key_error = raised"
        ]

        def get_error_key(dic, idx):
            return dic[idx]

        pars = Parsers()
        pars.parse_list_entry(test_root, raw)
        self.assertEqual(test_root["test_dict"][0], 1234)
        self.assertEqual(test_root["test_dict"][1], 12.34)
        self.assertEqual(test_root["test_dict"][2], True)
        self.assertEqual(test_root["test_dict"][3], False)
        self.assertEqual(test_root["test_dict"][4], "string")
        self.assertRaises(IndexError, get_error_key, test_root["test_dict"], 5)

    def test_json_section_parse_pass(self):
        pars = Parsers()
        test_root = dict()
        raw = [
            '  { "t_int": 1234,        ',
            '    "t_obj": {            ',
            '        "t_float": 12.34, ',
            '         "t_true": true   ',
            '     },                   ',
            '     "t_false": false,    ',
            '     "t_string": "string" ',
            '   }'
        ]
        test_root["json_pass"] = pars.parse_json_section(raw, "json_pass")

        self.assertEqual(
            test_root["json_pass"],
            dict(
                t_int=1234,
                t_obj=dict(
                    t_float=12.34,
                    t_true=True,
                ),
                t_false=False,
                t_string="string"
            )
        )

    def test_json_section_fail_silent(self):
        pars = Parsers(silent=True)
        raw = [
            '@json json_fail           ',
            '  { no__quotes: 1234,     ',
            '    "t_obj": {            ',
            '        "t_float": 12.34, ',
            '         "t_true": true   ',
            '     },                   ',
            '     "t_false": false,    ',
            '     "t_string": "string" ',
            '   }'
        ]
        self.assertEqual(pars.parse_json_section(raw, "json_fail"), {})
        self.assertEqual(pars.warnings[0], 'Parser error. Json parse error in "json_fail" section.')
        self.assertEqual(len(pars.warnings), 1)

    def test_json_section_fail_raise(self):
        pars = Parsers()
        raw = [
            '@json json_fail           ',
            '  { no__quotes: 1234,     ',
            '    "t_obj": {            ',
            '        "t_float": 12.34, ',
            '         "t_true": true   ',
            '     },                   ',
            '     "t_false": false,    ',
            '     "t_string": "string" ',
            '   }'
        ]
        with self.assertRaises(ParserException) as cm:
            pars.parse_json_section(raw, "json_fail")
        self.assertEqual(cm.exception.message, "Parser exception.Json parse error in \"json_fail\" section.")

    def test_json_parser_pass(self):
        pars = Parsers()
        test_root = dict()
        raw = [
            '@json json_pass           ',
            '  { "t_int": 1234,        ',
            '    "t_obj": {            ',
            '        "t_float": 12.34, ',
            '         "t_true": true   ',
            '     },                   ',
            '     "t_false": false,    ',
            '     "t_string": "string" ',
            '   }'
        ]
        pars.parse_json_entry(test_root, raw)
        self.assertEqual(
            test_root["json_pass"],
            dict(
                t_int=1234,
                t_obj=dict(
                    t_float=12.34,
                    t_true=True,
                ),
                t_false=False,
                t_string="string"
            )
        )

    def test_json_parser_fail_silent(self):
        pars = Parsers(silent=True)
        test_root = dict()
        raw = [
            '@json test_json           ',
            '  { no__quotes: 1234,        ',
            '    "t_obj": {            ',
            '        "t_float": 12.34, ',
            '         "t_true": true   ',
            '     },                   ',
            '     "t_false": false,    ',
            '     "t_string": "string" ',
            '   }'
        ]
        pars.parse_json_entry(test_root, raw)
        self.assertEqual(test_root["test_json"], {})
        self.assertEqual(pars.warnings[0], 'Parser error. Json entry parse error for entry: test_json.')
        self.assertEqual(len(pars.warnings), 1)

    def test_json_parser_fail_raise(self):
        pars = Parsers()
        test_root = dict()
        raw = [
            '@json test_json           ',
            '  { no__quotes: 1234,        ',
            '    "t_obj": {            ',
            '        "t_float": 12.34, ',
            '         "t_true": true   ',
            '     },                   ',
            '     "t_false": false,    ',
            '     "t_string": "string" ',
            '   }'
        ]
        with self.assertRaises(ParserException) as cm:
            pars.parse_json_entry(test_root, raw)
        self.assertEqual(cm.exception.message, "Parser exception.Json entry parse error for entry: test_json.")

    def test_symlink_list(self):
        raw = [
            "source_root/dir1 > dest_root/dir1",
            "source_root/dir2/dir21 > dest_root/dir2/dir21",
            "source_root/dir3 > dest_root/dir3"
        ]
        parsed = Parsers.parse_symlink_list(raw)
        self.assertEqual(parsed,
                         [
                             {'dest': 'dest_root/dir1', 'source': 'source_root/dir1'},
                             {'dest': 'dest_root/dir2/dir21', 'source': 'source_root/dir2/dir21'},
                             {'dest': 'dest_root/dir3', 'source': 'source_root/dir3'}
                         ])



from setuptools import setup

setup(name='big_config',
      version='0.3.1',
      description='Handles a more complex configuration file structure',
      url='http://github.com/vladblindu/',
      author='Vlad Blindu',
      author_email='vlad.blindu@bbsmedia.ro',
      license='BSD',
      packages=['big_config'],
      zip_safe=False)



import json

import os
from cerberus import Validator

from constants import *
from parsers import Parsers, ParserException


class ConfigException(Exception):
    def __init__(self, message):
        self.message = message
        Exception.__init__(self, self.message)


class Config(dict):
    def __init__(self, config_file="", logger=None,
                 silent=False, schema=None):
        super(Config, self).__init__(self)
        self.__warnings = []
        self.__logger = logger
        self.__silent = silent
        self.__schema = schema
        self.__parsers = Parsers(silent=silent)
        if config_file:
            self.initialize(config_file)

    def initialize(self, config_file):
        content = self.read_config(config_file)
        if content:
            raw_data = self.pre_parse(content)
            if len(raw_data):
                data = self.parse(raw_data)
                if len(data):
                    if self.__schema:
                        if not self.validate(data):
                            return {}
                    self.update(data)
                    return self
        return {}

    def log(self, message):
        if not self.__silent:
            if self.__logger:
                self.__logger.error(message)
            raise ConfigException(message)
        self.__warnings.append(message)

    def pre_parse(self, content):
        raw_data = []
        section_name = ""
        conf_arr = content.split("\n")
        if len(conf_arr) < 2:
            self.log("Config Exception. Corrupted or missing config file content.")
            return []
        for idx in range(0, len(conf_arr)):
            line = conf_arr[idx]
            if not line.strip() or line.strip()[0] == COMMENT_MARK:
                continue
            if line.strip()[0] == SECTION_IN and line.strip()[-1] == SECTION_OUT:
                section_name = line.strip()[1:-1]
                if section_name[0] == PARSER_SECTION:
                    bits = section_name.split(BLANK)
                    section_name = bits[1]
                    try:
                        section_type = SECTION_TYPES[bits[0][1:]]
                    except KeyError:
                        self.log("Config Exception. Key error raised in section type: %s" % bits[0][1:])
                        section_name = ""
                        continue
                else:
                    section_type = 0
                raw_data.append(dict(name=section_name, type=section_type, body=[]))
                continue
            if not section_name:
                continue
            raw_data[-1]["body"].append(line)
        if not raw_data or not len(raw_data):
            self.log("Config error. Could not parse file content. Got empty dict")
            return []
        return raw_data

    def read_config(self, config_file):

        if not os.path.isfile(config_file):
            self.log("Config error. %s missing configuration file." % config_file)
            return ""
        try:
            conf = open(config_file, "r")
            content = conf.read()
        except IOError as err:
            self.log("Configuration error for file: %s -  %s." %
                     (config_file, err.strerror or err.message))
            return ""
        if conf and not conf.closed:
            conf.close()
        if not content:
            self.log("Configuration error. Configuration file %s is empty." % config_file)
        return content

    def parse(self, raw_data):
        output = dict()
        for item in raw_data:
            try:
                if item["type"] == 1:
                    output[item["name"]] = self.__parsers.parse_dir_list(item["body"])
                    continue

                if item["type"] == 2:
                    output[item["name"]] = self.__parsers.parse_symlink_list(item["body"])
                    continue

                if item["type"] == 3:
                    output[item["name"]] = self.__parsers.parse_json_section(item["body"], item["name"])
                    continue

                output[item["name"]] = self.__parsers.parse(item["body"])
            except ParserException as err:
                self.log(err.message)
            self.__warnings += self.__parsers.warnings
        if not len(output):
            message = "Configuration error. Config dict is empty"
            self.log(message)
        return output

    def validate(self, conf):
        validator = Validator()
        if validator(conf, self.__schema):
            return True
        self.log("Configuration validation error(s): \n" +
                 json.dumps(validator.errors, indent=3))
        return False

    def warnings(self):
        return self.__warnings

    def status(self):
        return dict(
            _status="OK" if len(self.__warnings) == 0 else "ERR",
            _errors=self.warnings
        )



import os

from big_config.constants import COMMENT_MARK


class PathAsList(list):
    """
        inherits from list and adds specific methods
        for a model defining a path with a list of dirs
    """

    def __init__(self, parent=""):
        super(PathAsList, self).__init__(self)
        if parent:
            self.append(parent)

    # go down one level
    def down(self, elem):
        self.append(elem)

    # go up n levels
    def up(self, levels=1):
        while levels < 0:
            del self[-1]
            levels += 1

    # convert list to path string
    def to_path(self):
        return os.path.join(*self) if len(self) else ""


class Step(list):
    """
        inherits from list and adds specific methods to describe
        a list of indent increment steps in a list of paths
    """

    def __init__(self):
        super(Step, self).__init__(self)
        self.append(0)

    # compare current vs last indent value
    def inspect(self, indent):
        if indent > self[-1]:
            self.append(indent)
            return 1
        elif indent == self[-1]:
            return 0
        else:
            back_steps = 0
            while indent < self[-1]:
                del self[-1]
                back_steps -= 1
            return back_steps


class PathsList(list):
    def __init__(self, raw, parent=""):
        super(PathsList, self).__init__(self)
        # set the parent
        self.__current = PathAsList(parent)
        if raw and len(raw):
            self.add(raw)

    @staticmethod
    def get_indent(line):
        return len(line) - len(line.lstrip())

    # clean and prepare raw list for processing
    @staticmethod
    def __prepare(raw):
        # declare an empty output
        output = []
        # declare a step instance
        step = Step()
        # strip comments and blank lines at the beginning
        while not raw[0].strip() or raw[0].strip()[0] == COMMENT_MARK:
            del raw[0]
        # detect a general indent
        ind0 = PathsList.get_indent(raw[0])

        for elem in raw:
            # if blank or comment ignore
            if not elem.strip() or elem.strip()[0] == COMMENT_MARK:
                continue
            # get the current line's indent
            ind = PathsList.get_indent(elem) - ind0
            # append to output a tuple containing the step and the clean string
            output.append((step.inspect(ind), elem.strip()))
        return output

    def __add_elem(self, elem, last):
        # if step goes deeper add the last element to the current path list
        if elem[0] == 1:
            self.__current.down(last)
        # if step goes up add the last element to the current path list
        if elem[0] < 0:
            self.__current.up(elem[0])
        # append the resulting path
        self.append(os.path.join(self.__current.to_path(), elem[1]))

    def add(self, raw):
        data = self.__prepare(raw)
        # keep the last element
        last = None
        for elem in data:
            self.__add_elem(elem, last)
            # update the last element
            last = elem[1]
